<?php

namespace FiloBlu\MappConnectorExtension\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;

/**
 * Class MappAdditionalData
 * @package FiloBlu\Newsletter\Observer
 */
class MappAdditionalData implements ObserverInterface
{
    /**
     * XML PATHS
     */
    const MAPPCONNECTOR_EXT_DOB_FORMAT = "mappconnector/extension/dob_format";

    const DEFAULT_INPUT_DOB_FORMAT = "Y-m-d";
    const DEFAULT_OUTPUT_DOB_FORMAT = "Y-m-d";

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var \FiloBlu\Newsletter\Model\SubscriberInfoFactory
     */
    protected $subscriberInfoFactory;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $config;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $extConfig;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * MappAdditionalData constructor.
     * @param \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \FiloBlu\Newsletter\Model\SubscriberInfoFactory $subscriberInfoFactory,
        \Psr\Log\LoggerInterface $logger,
        \FiloBlu\MappConnector\Model\Config $config,
        \Magento\Framework\App\Config\ScopeConfigInterface $extConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    )
    {
        $this->subscriberInfoFactory = $subscriberInfoFactory;
        $this->logger = $logger;
        $this->config = $config;
        $this->extConfig = $extConfig;
        $this->storeManager = $storeManager;
    }

    /**
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        try {
            $subscriber_object = $observer->getResult();
            $result = $subscriber_object->getSubscriberData();
            $subinfo = $this->subscriberInfoFactory->create();

            $nl_additional_data = [
                'subscriber_id' => '',
                'firstname' => '',
                'lastname' => '',
                'dob' => '',
                'country' => '',
                'gender' => '',
                'collections' => '',
                'source' => '',
                'language' => '',
                'website_name' => '',
                'birthday' => '',
                'phone' => '',
                'created_at' => '',
                'updated_at' => ''
            ];

            $newsletter_additional_data = $subinfo->load($result['subscriberid']);

            if (count($newsletter_additional_data->getData()) > 0) {
                $nl_additional_data = $newsletter_additional_data->getData();

                if (!empty($nl_additional_data['dob'])) {
                    //$nl_additional_data['dob'] = date('Y-m-d', strtotime($nl_additional_data['dob'] . ' 00:00:00'));
                    $nl_additional_data['dob'] = $this->formatDate($nl_additional_data['dob']);
                }

                //Get mapped gender
                $nl_additional_data['gender'] = $this->config->getNormalizedGender($nl_additional_data['gender']);

                $lan = explode('_', $nl_additional_data['language']);
                $nl_additional_data['language'] = $lan[0];
            }

            $result = array_merge($result, $nl_additional_data);

            unset(
                $result['subscriber_id'],
                $result['subscriberid']
            );

            $subscriber_object->setSubscriberData($result);
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
        }

    }

    /*
     * This function will try to read the date from a string
     * Check filoblu_newsletter_subscriber_info -> dob column
     */
    private function formatDate($string_date){

        $input_format = $this->extConfig->getValue(
            self::MAPPCONNECTOR_EXT_DOB_FORMAT,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE,
            $this->storeManager->getStore()->getId()
        );
        if (!$input_format){
            $input_format = self::DEFAULT_INPUT_DOB_FORMAT;
        }

        try {
            $date = date_create_from_format($input_format, $string_date);
            if ($date){
                return date_format($date, self::DEFAULT_OUTPUT_DOB_FORMAT);
            }
        }catch(\Exception $e) {}
        return null;
    }

}